@@ -13,20 +13,29 @@ suppress_warnings = ["image.nonlocal_uri"]
 # -- General configuration ----------------------------------------------------
 
 extensions = [
+    "myst_parser",
     "notfound.extension",
     "sphinx.ext.autodoc",
     "sphinx.ext.autodoc.typehints",
+    "sphinx.ext.napoleon",
     "sphinx.ext.doctest",
     "sphinx.ext.intersphinx",
     "sphinx.ext.viewcode",
     "sphinxcontrib.mermaid",
 ]
 
+myst_enable_extensions = [
+    "colon_fence",
+    "smartquotes",
+    "deflist",
+]
+mermaid_init_js = "mermaid.initialize({startOnLoad:true,theme:'neutral'});"
+
 # Add any paths that contain templates here, relative to this directory.
 templates_path = ["_templates"]
 
 # The suffix of source filenames.
-source_suffix = ".rst"
+source_suffix = [".rst", ".md"]
 
 # The master toctree document.
 master_doc = "index"
@@ -45,6 +54,9 @@ release = metadata.version("structlog")
 # The short X.Y version.
 version = release.rsplit(".", 1)[0]
 
+if "dev" in release:
+    release = version = "UNRELEASED"
+
 exclude_patterns = ["_build"]
 
 # The reST default role (used for this markup: `text`) to use for all
@@ -52,27 +64,20 @@ exclude_patterns = ["_build"]
 default_role = "any"
 
 nitpick_ignore = [
-    ("py:class", "BinaryIO"),
+    ("py:class", "Context"),
+    ("py:class", "EventDict"),
     ("py:class", "ILogObserver"),
     ("py:class", "PlainFileObserver"),
+    ("py:class", "Processor"),
+    ("py:class", "Styles"),
+    ("py:class", "WrappedLogger"),
     ("py:class", "structlog.threadlocal.TLLogger"),
-    ("py:class", "TextIO"),
-    ("py:class", "TLLogger"),
-    ("py:class", "traceback"),
-    ("py:class", "structlog._base.BoundLoggerBase"),
-    ("py:class", "structlog.dev._Styles"),
-    ("py:class", "structlog.types.EventDict"),
-    ("py:obj", "sync_bl"),
-    ("py:obj", "entries"),
+    ("py:class", "structlog.typing.EventDict"),
 ]
 
 # If true, '()' will be appended to :func: etc. cross-reference text.
 add_function_parentheses = True
 
-# If true, the current module name will be prepended to all description
-# unit titles (such as .. function::).
-# add_module_names = True
-
 # Move type hints into the description block, instead of the func definition.
 autodoc_typehints = "description"
 autodoc_typehints_description_target = "documented"
@@ -85,6 +90,7 @@ html_theme = "furo"
 html_theme_options = {}
 html_logo = "_static/structlog_logo_small_transparent.png"
 html_static_path = ["_static"]
+
 htmlhelp_basename = "structlogdoc"
 
 _logo = (
@@ -147,11 +153,14 @@ epub_copyright = copyright
 
 # GitHub has rate limits
 linkcheck_ignore = [
-    r"https://github.com/.*/(issues|pull)/\d+",
+    r"https://github.com/.*/(issues|pull|compare)/\d+",
     r"https://twitter.com/.*",
 ]
 
 # Twisted's trac tends to be slow
 linkcheck_timeout = 300
 
-intersphinx_mapping = {"python": ("https://docs.python.org/3", None)}
+intersphinx_mapping = {
+    "python": ("https://docs.python.org/3", None),
+    "rich": ("https://rich.readthedocs.io/en/stable/", None),
+}
