@@ -1,8 +1,8 @@
+# SPDX-License-Identifier: MIT OR Apache-2.0
 # This file is dual licensed under the terms of the Apache License, Version
 # 2.0, and the MIT License.  See the LICENSE file in the root of this
 # repository for complete details.
 
-
 import sys
 
 import pytest
@@ -85,7 +85,7 @@ class TestFindFirstAppFrameAndName:
         assert (f1, "?") == (f, n)
 
 
-@pytest.fixture
+@pytest.fixture()
 def exc_info():
     """
     Fake a valid exc_info.
