@@ -3,6 +3,8 @@
 # 2.0, and the MIT License.  See the LICENSE file in the root of this
 # repository for complete details.
 
+from __future__ import annotations
+
 import datetime
 import functools
 import inspect
@@ -15,7 +17,6 @@ import sys
 import threading
 
 from io import StringIO
-from typing import Any, Dict, List, Optional, Set
 
 import pytest
 
@@ -28,10 +29,13 @@ from structlog._utils import get_processname
 from structlog.processors import (
     CallsiteParameter,
     CallsiteParameterAdder,
+    EventRenamer,
     ExceptionPrettyPrinter,
+    ExceptionRenderer,
     JSONRenderer,
     KeyValueRenderer,
     LogfmtRenderer,
+    MaybeTimeStamper,
     StackInfoRenderer,
     TimeStamper,
     UnicodeDecoder,
@@ -42,8 +46,9 @@ from structlog.processors import (
 )
 from structlog.stdlib import ProcessorFormatter
 from structlog.threadlocal import wrap_dict
-from structlog.types import EventDict
+from structlog.typing import EventDict
 from tests.additional_frame import additional_frame
+from tests.utils import CustomError
 
 
 try:
@@ -282,11 +287,9 @@ class TestLogfmtRenderer:
             "invalid key": "somevalue",
         }
 
-        with pytest.raises(ValueError) as e:
+        with pytest.raises(ValueError, match='Invalid key: "invalid key"'):
             LogfmtRenderer()(None, None, event_dict)
 
-        assert 'Invalid key: "invalid key"' == e.value.args[0]
-
 
 class TestJSONRenderer:
     def test_renders_json(self, event_dict):
@@ -305,9 +308,10 @@ class TestJSONRenderer:
         """
         Our fallback handling handles properly ThreadLocalDictWrapper values.
         """
-        s = json.dumps(
-            wrap_dict(dict)({"a": 42}), default=_json_fallback_handler
-        )
+        with pytest.deprecated_call():
+            d = wrap_dict(dict)
+
+        s = json.dumps(d({"a": 42}), default=_json_fallback_handler)
 
         assert '{"a": 42}' == s
 
@@ -362,11 +366,11 @@ class TestTimeStamper:
         A asking for a UNIX timestamp with a timezone that's not UTC raises a
         ValueError.
         """
-        with pytest.raises(ValueError) as e:
+        with pytest.raises(
+            ValueError, match="UNIX timestamps are always UTC."
+        ):
             TimeStamper(utc=False)
 
-        assert "UNIX timestamps are always UTC." == e.value.args[0]
-
     def test_inserts_utc_unix_timestamp_by_default(self):
         """
         Per default a float UNIX timestamp is used.
@@ -437,47 +441,71 @@ class TestTimeStamper:
             None, None, {}
         )
 
-    @pytest.mark.parametrize(
-        ("utc", "expect"),
-        [
-            (True, "1980-03-25T16:00:00Z"),
-            (False, "1980-03-25T17:00:00"),
-        ],
-    )
-    def test_apply_freezegun_after_instantiation(self, utc, expect):
+    def test_apply_freezegun_after_instantiation(self):
         """
-        Instantiate TimeStamper after mocking datetime
+        Freezing time after instantiation of TimeStamper works.
         """
-        ts = TimeStamper(fmt="iso", utc=utc)
+        ts = TimeStamper(fmt="iso", utc=False)
+
         with freeze_time("1980-03-25 16:00:00", tz_offset=1):
             d = ts(None, None, {})
-            assert expect == d["timestamp"]
+
+            assert "1980-03-25T17:00:00" == d["timestamp"]
+
+
+class TestMaybeTimeStamper:
+    def test_overwrite(self):
+        """
+        If there is a timestamp, leave it.
+        """
+        mts = MaybeTimeStamper()
+
+        assert {"timestamp": 42} == mts(None, None, {"timestamp": 42})
+
+    def test_none(self):
+        """
+        If there is no timestamp, add one.
+        """
+        mts = MaybeTimeStamper()
+
+        assert "timestamp" in mts(None, None, {})
 
 
 class TestFormatExcInfo:
+    def test_custom_formatter(self):
+        """
+        The exception formatter can be changed.
+        """
+        formatter = ExceptionRenderer(lambda _: "There is no exception!")
+
+        try:
+            raise CustomError("test")
+        except CustomError as e:
+            exc = e
+
+        assert formatter(None, None, {"exc_info": exc}) == {
+            "exception": "There is no exception!"
+        }
+
     @pytest.mark.parametrize("ei", [False, None, ""])
     def test_nop(self, ei):
         """
         If exc_info is falsey, only remove the key.
         """
-        assert {} == format_exc_info(None, None, {"exc_info": ei})
+        assert {} == ExceptionRenderer()(None, None, {"exc_info": ei})
 
     def test_nop_missing(self):
         """
         If event dict doesn't contain exc_info, do nothing.
         """
-        assert {} == format_exc_info(None, None, {})
+        assert {} == ExceptionRenderer()(None, None, {})
 
-    def test_formats_tuple(self, monkeypatch):
+    def test_formats_tuple(self):
         """
         If exc_info is a tuple, it is used.
         """
-        monkeypatch.setattr(
-            structlog.processors,
-            "_format_exception",
-            lambda exc_info: exc_info,
-        )
-        d = format_exc_info(None, None, {"exc_info": (None, None, 42)})
+        formatter = ExceptionRenderer(lambda exc_info: exc_info)
+        d = formatter(None, None, {"exc_info": (None, None, 42)})
 
         assert {"exception": (None, None, 42)} == d
 
@@ -490,64 +518,85 @@ class TestFormatExcInfo:
         try:
             raise ValueError("test")
         except ValueError:
-            d = format_exc_info(None, None, {"exc_info": True})
+            d = ExceptionRenderer()(None, None, {"exc_info": True})
 
         assert "exc_info" not in d
-        assert 'raise ValueError("test")\nValueError: test' in d["exception"]
+        assert 'raise ValueError("test")' in d["exception"]
+        assert "ValueError: test" in d["exception"]
 
-    def test_exception_on_py3(self, monkeypatch):
+    def test_exception(self):
         """
-        Passing exceptions as exc_info is valid on Python 3.
+        Passing exceptions as exc_info is valid.
         """
-        monkeypatch.setattr(
-            structlog.processors,
-            "_format_exception",
-            lambda exc_info: exc_info,
-        )
+        formatter = ExceptionRenderer(lambda exc_info: exc_info)
+
         try:
             raise ValueError("test")
         except ValueError as e:
-            d = format_exc_info(None, None, {"exc_info": e})
-
-            assert {"exception": (ValueError, e, e.__traceback__)} == d
+            exc = e
         else:
             pytest.fail("Exception not raised.")
 
+        assert {
+            "exception": (ValueError, exc, exc.__traceback__)
+        } == formatter(None, None, {"exc_info": exc})
+
     def test_exception_without_traceback(self):
         """
         If an Exception is missing a traceback, render it anyway.
         """
-        rv = format_exc_info(
+        rv = ExceptionRenderer()(
             None, None, {"exc_info": Exception("no traceback!")}
         )
 
         assert {"exception": "Exception: no traceback!"} == rv
 
+    def test_format_exception(self):
+        """
+        "format_exception" is the "ExceptionRenderer" with default settings.
+        """
+        try:
+            raise ValueError("test")
+        except ValueError as e:
+            a = format_exc_info(None, None, {"exc_info": e})
+            b = ExceptionRenderer()(None, None, {"exc_info": e})
+
+        assert a == b
+
+    @pytest.mark.parametrize("ei", [True, (None, None, None)])
+    def test_no_exception(self, ei):
+        """
+        A missing exception does not blow up.
+        """
+        assert {"exception": "MISSING"} == format_exc_info(
+            None, None, {"exc_info": ei}
+        )
+
 
 class TestUnicodeEncoder:
     def test_encodes(self):
         """
         Unicode strings get encoded (as UTF-8 by default).
         """
-        ue = UnicodeEncoder()
+        e = UnicodeEncoder()
 
-        assert {"foo": b"b\xc3\xa4r"} == ue(None, None, {"foo": "b\xe4r"})
+        assert {"foo": b"b\xc3\xa4r"} == e(None, None, {"foo": "b\xe4r"})
 
     def test_passes_arguments(self):
         """
         Encoding options are passed into the encoding call.
         """
-        ue = UnicodeEncoder("latin1", "xmlcharrefreplace")
+        e = UnicodeEncoder("latin1", "xmlcharrefreplace")
 
-        assert {"foo": b"&#8211;"} == ue(None, None, {"foo": "\u2013"})
+        assert {"foo": b"&#8211;"} == e(None, None, {"foo": "\u2013"})
 
     def test_bytes_nop(self):
         """
         If the string is already bytes, don't do anything.
         """
-        ue = UnicodeEncoder()
+        e = UnicodeEncoder()
 
-        assert {"foo": b"b\xc3\xa4r"} == ue(None, None, {"foo": b"b\xc3\xa4r"})
+        assert {"foo": b"b\xc3\xa4r"} == e(None, None, {"foo": b"b\xc3\xa4r"})
 
 
 class TestUnicodeDecoder:
@@ -681,7 +730,7 @@ class TestExceptionPrettyPrinter:
         assert "XXX" in sio.getvalue()
 
 
-@pytest.fixture
+@pytest.fixture()
 def sir():
     return StackInfoRenderer()
 
@@ -704,10 +753,25 @@ class TestStackInfoRenderer:
         assert "stack" in ed
 
     def test_renders_correct_stack(self, sir):
+        """
+        The rendered stack is correct.
+        """
         ed = sir(None, None, {"stack_info": True})
 
         assert 'ed = sir(None, None, {"stack_info": True})' in ed["stack"]
 
+    def test_additional_ignores(self):
+        """
+        Filtering of names works.
+        """
+        sir = StackInfoRenderer(["tests.additional_frame"])
+
+        ed = additional_frame(
+            functools.partial(sir, None, None, {"stack_info": True})
+        )
+
+        assert "additional_frame.py" not in ed["stack"]
+
 
 class TestFigureOutExcInfo:
     @pytest.mark.parametrize("true_value", [True, 1, 1.1])
@@ -765,37 +829,33 @@ class TestCallsiteParameterAdder:
             "determine the callsite for async calls."
         )
     )
-    @pytest.mark.asyncio
+    @pytest.mark.asyncio()
     async def test_async(self) -> None:
         """
         Callsite information for async invocations are correct.
         """
-        try:
-            string_io = StringIO()
+        string_io = StringIO()
 
-            class StingIOLogger(structlog.PrintLogger):
-                def __init__(self):
-                    super().__init__(file=string_io)
+        class StingIOLogger(structlog.PrintLogger):
+            def __init__(self):
+                super().__init__(file=string_io)
 
-            processor = self.make_processor(None, ["concurrent", "threading"])
-            structlog.configure(
-                processors=[processor, JSONRenderer()],
-                logger_factory=StingIOLogger,
-                wrapper_class=structlog.stdlib.AsyncBoundLogger,
-                cache_logger_on_first_use=True,
-            )
+        processor = self.make_processor(None, ["concurrent", "threading"])
+        structlog.configure(
+            processors=[processor, JSONRenderer()],
+            logger_factory=StingIOLogger,
+            wrapper_class=structlog.stdlib.AsyncBoundLogger,
+            cache_logger_on_first_use=True,
+        )
 
-            logger = structlog.stdlib.get_logger()
+        logger = structlog.stdlib.get_logger()
 
-            callsite_params = self.get_callsite_parameters()
-            await logger.info("baz")
+        callsite_params = self.get_callsite_parameters()
+        await logger.info("baz")
 
-            assert {"event": "baz", **callsite_params} == json.loads(
-                string_io.getvalue()
-            )
-
-        finally:
-            structlog.reset_defaults()
+        assert {"event": "baz", **callsite_params} == json.loads(
+            string_io.getvalue()
+        )
 
     def test_additional_ignores(self, monkeypatch: pytest.MonkeyPatch) -> None:
         """
@@ -828,7 +888,7 @@ class TestCallsiteParameterAdder:
         assert expected == actual
 
     @pytest.mark.parametrize(
-        "origin, parameter_strings",
+        ("origin", "parameter_strings"),
         itertools.product(
             ["logging", "structlog"],
             [
@@ -844,7 +904,7 @@ class TestCallsiteParameterAdder:
     def test_processor(
         self,
         origin: str,
-        parameter_strings: Optional[Set[str]],
+        parameter_strings: set[str] | None,
     ):
         """
         The correct callsite parameters are added to event dictionaries.
@@ -891,7 +951,7 @@ class TestCallsiteParameterAdder:
         assert expected == actual
 
     @pytest.mark.parametrize(
-        "setup, origin, parameter_strings",
+        ("setup", "origin", "parameter_strings"),
         itertools.product(
             ["common-without-pre", "common-with-pre", "shared", "everywhere"],
             ["logging", "structlog"],
@@ -909,7 +969,7 @@ class TestCallsiteParameterAdder:
         self,
         setup: str,
         origin: str,
-        parameter_strings: Optional[Set[str]],
+        parameter_strings: set[str] | None,
     ) -> None:
         """
         Logging output contains the correct callsite parameters.
@@ -952,7 +1012,7 @@ class TestCallsiteParameterAdder:
             callsite_params = self.get_callsite_parameters()
             logger.info(test_message)
         elif origin == "structlog":
-            ctx: Dict[str, Any] = {}
+            ctx = {}
             bound_logger = BoundLogger(
                 logger,
                 [*common_processors, ProcessorFormatter.wrap_for_formatter],
@@ -981,45 +1041,50 @@ class TestCallsiteParameterAdder:
     @classmethod
     def make_processor(
         cls,
-        parameter_strings: Optional[Set[str]],
-        additional_ignores: Optional[List[str]] = None,
+        parameter_strings: set[str] | None,
+        additional_ignores: list[str] | None = None,
     ) -> CallsiteParameterAdder:
         """
         Creates a ``CallsiteParameterAdder`` with parameters matching the
         supplied ``parameter_strings`` values and with the supplied
         ``additional_ignores`` values.
 
-        :param parameter_strings:
-            Strings for which corresponding ``CallsiteParameters`` should be
-            included in the resulting ``CallsiteParameterAdded``.
-        :param additional_ignores:
-            Used as ``additional_ignores`` for the resulting
-            ``CallsiteParameterAdded``.
+        Arguments:
+
+            parameter_strings:
+                Strings for which corresponding ``CallsiteParameters`` should
+                be included in the resulting ``CallsiteParameterAdded``.
+
+            additional_ignores:
+
+                Used as ``additional_ignores`` for the resulting
+                ``CallsiteParameterAdded``.
         """
         if parameter_strings is None:
             return CallsiteParameterAdder(
                 additional_ignores=additional_ignores
             )
-        else:
-            parameters = cls.filter_parameters(parameter_strings)
-            return CallsiteParameterAdder(
-                parameters=parameters,
-                additional_ignores=additional_ignores,
-            )
+
+        parameters = cls.filter_parameters(parameter_strings)
+        return CallsiteParameterAdder(
+            parameters=parameters,
+            additional_ignores=additional_ignores,
+        )
 
     @classmethod
     def filter_parameters(
-        cls, parameter_strings: Optional[Set[str]]
-    ) -> Set[CallsiteParameter]:
+        cls, parameter_strings: set[str] | None
+    ) -> set[CallsiteParameter]:
         """
         Returns a set containing all ``CallsiteParameter`` members with values
         that are in ``parameter_strings``.
 
-        :param parameter_strings:
-            The parameters strings for which corresponding
-            ``CallsiteParameter`` members should be
-            returned. If this value is `None` then all
-            ``CallsiteParameter`` will be returned.
+        Arguments:
+
+            parameter_strings:
+                The parameters strings for which corresponding
+                ``CallsiteParameter`` members should be returned. If this value
+                is `None` then all ``CallsiteParameter`` will be returned.
         """
         if parameter_strings is None:
             return cls._all_parameters
@@ -1031,15 +1096,17 @@ class TestCallsiteParameterAdder:
 
     @classmethod
     def filter_parameter_dict(
-        cls, input: Dict[str, Any], parameter_strings: Optional[Set[str]]
-    ) -> Dict[str, Any]:
+        cls, input: dict[str, object], parameter_strings: set[str] | None
+    ) -> dict[str, object]:
         """
         Returns a dictionary that is equivalent to ``input`` but with all keys
         not in ``parameter_strings`` removed.
 
-        :param parameter_strings:
-            The keys to keep in the dictionary, if this value is ``None`` then
-            all keys matching ``cls.parameter_strings`` are kept.
+        Arguments:
+
+            parameter_strings:
+                The keys to keep in the dictionary, if this value is ``None``
+                then all keys matching ``cls.parameter_strings`` are kept.
         """
         if parameter_strings is None:
             parameter_strings = cls.parameter_strings
@@ -1050,14 +1117,16 @@ class TestCallsiteParameterAdder:
         }
 
     @classmethod
-    def get_callsite_parameters(cls, offset: int = 1) -> Dict[str, Any]:
+    def get_callsite_parameters(cls, offset: int = 1) -> dict[str, object]:
         """
         This function creates dictionary of callsite parameters for the line
         that is ``offset`` lines after the invocation of this function.
 
-        :param offset:
-            The amount of lines after the invocation of this function that
-            callsite parameters should be generated for.
+        Arguments:
+
+            offset:
+                The amount of lines after the invocation of this function that
+                callsite parameters should be generated for.
         """
         frame_info = inspect.stack()[1]
         frame_traceback = inspect.getframeinfo(frame_info[0])
@@ -1074,3 +1143,33 @@ class TestCallsiteParameterAdder:
             "process": os.getpid(),
             "process_name": get_processname(),
         }
+
+
+class TestRenameKey:
+    def test_rename_once(self):
+        """
+        Renaming event to something else works.
+        """
+        assert {"msg": "hi", "foo": "bar"} == EventRenamer("msg")(
+            None, None, {"event": "hi", "foo": "bar"}
+        )
+
+    def test_rename_twice(self):
+        """
+        Renaming both from and to `event` works.
+        """
+        assert {
+            "msg": "hi",
+            "event": "fabulous",
+            "foo": "bar",
+        } == EventRenamer("msg", "_event")(
+            None, None, {"event": "hi", "foo": "bar", "_event": "fabulous"}
+        )
+
+    def test_replace_by_key_is_optional(self):
+        """
+        The key that is renamed to `event` doesn't have to exist.
+        """
+        assert {"msg": "hi", "foo": "bar"} == EventRenamer("msg", "missing")(
+            None, None, {"event": "hi", "foo": "bar"}
+        )
