@@ -3,6 +3,8 @@
 # 2.0, and the MIT License.  See the LICENSE file in the root of this
 # repository for complete details.
 
+from __future__ import annotations
+
 import json
 import logging
 import logging.config
@@ -10,9 +12,10 @@ import os
 import sys
 
 from io import StringIO
-from typing import Any, Callable, Collection, Dict, Optional, Set
+from typing import Any, Callable, Collection
 
 import pytest
+import pytest_asyncio
 
 from pretend import call_recorder, stub
 
@@ -21,8 +24,8 @@ from structlog import (
     ReturnLogger,
     configure,
     get_context,
-    reset_defaults,
 )
+from structlog._config import _CONFIG
 from structlog._log_levels import _NAME_TO_LEVEL, CRITICAL, WARN
 from structlog.dev import ConsoleRenderer
 from structlog.exceptions import DropEvent
@@ -40,10 +43,11 @@ from structlog.stdlib import (
     add_logger_name,
     filter_by_level,
     get_logger,
+    recreate_defaults,
     render_to_log_kwargs,
 )
 from structlog.testing import CapturedCall
-from structlog.types import BindableLogger, EventDict
+from structlog.typing import BindableLogger, EventDict
 
 from .additional_frame import additional_frame
 
@@ -105,7 +109,11 @@ class TestLoggerFactory:
         )
 
     def test_deduces_correct_caller(self):
+        """
+        It will find the correct caller.
+        """
         logger = _FixedFindCallerLogger("test")
+
         file_name, line_number, func_name = logger.findCaller()[:3]
 
         assert file_name == os.path.realpath(__file__)
@@ -121,21 +129,25 @@ class TestLoggerFactory:
         assert "testing, is_, fun" in stack_info
 
     def test_no_stack_info_by_default(self):
+        """
+        If we don't ask for stack_info, it won't be returned.
+        """
         logger = _FixedFindCallerLogger("test")
         testing, is_, fun, stack_info = logger.findCaller()
 
         assert None is stack_info
 
-    def test_find_caller(self, monkeypatch):
+    def test_find_caller(self, caplog):
+        """
+        The caller is found.
+        """
         logger = LoggerFactory()()
-        log_handle = call_recorder(lambda x: None)
-        monkeypatch.setattr(logger, "handle", log_handle)
-        logger.error("Test")
-        log_record = log_handle.calls[0].args[0]
 
-        assert log_record.funcName == "test_find_caller"
-        assert log_record.name == __name__
-        assert log_record.filename == os.path.basename(__file__)
+        logger.error("Test")
+
+        assert caplog.text.startswith(
+            "ERROR    tests.test_stdlib:test_stdlib.py"
+        )
 
     def test_sets_correct_logger(self):
         """
@@ -234,7 +246,7 @@ class TestBoundLogger:
         assert bound_logger_attribute == stdlib_logger_attribute
 
     @pytest.mark.parametrize(
-        "method_name,method_args",
+        ("method_name", "method_args"),
         [
             ("addHandler", [None]),
             ("removeHandler", [None]),
@@ -337,6 +349,66 @@ class TestBoundLogger:
 
         assert {} == get_context(bl)
 
+    @pytest.mark.parametrize(
+        "meth", ["debug", "info", "warning", "error", "critical"]
+    )
+    async def test_async_log_methods(self, meth, cl):
+        """
+        Async methods log async.
+        """
+        bl = build_bl(cl, processors=[])
+
+        await getattr(bl, f"a{meth}")("Async!")
+
+        assert [
+            CapturedCall(method_name=meth, args=(), kwargs={"event": "Async!"})
+        ] == cl.calls
+
+    async def test_alog(self, cl):
+        """
+        Alog logs async at the correct level.
+        """
+        bl = build_bl(cl, processors=[])
+
+        await bl.alog(logging.INFO, "foo %s", "bar")
+
+        assert [
+            CapturedCall(
+                method_name="info",
+                args=(),
+                kwargs={"positional_args": ("bar",), "event": "foo %s"},
+            )
+        ] == cl.calls
+
+    async def test_aexception_exc_info_true(self, cl):
+        """
+        aexception passes current exc_info into dispatch.
+        """
+        bl = build_bl(cl, processors=[])
+
+        try:
+            raise ValueError(42)
+        except ValueError as e:
+            await bl.aexception("oops")
+            exc = e
+
+        (cc,) = cl.calls
+
+        assert isinstance(cc[2]["exc_info"], tuple)
+        assert exc == cc[2]["exc_info"][1]
+
+    async def test_aexception_exc_info_explicit(self, cl):
+        """
+        In aexception, if exc_info isn't missing or True, leave it be.
+        """
+        bl = build_bl(cl, processors=[])
+
+        obj = object()
+
+        await bl.aexception("ooops", exc_info=obj)
+
+        assert obj is cl.calls[0].kwargs["exc_info"]
+
 
 class TestPositionalArgumentsFormatter:
     def test_formats_tuple(self):
@@ -404,7 +476,7 @@ class TestPositionalArgumentsFormatter:
 
 
 class TestAddLogLevelNumber:
-    @pytest.mark.parametrize("level, number", _NAME_TO_LEVEL.items())
+    @pytest.mark.parametrize(("level", "number"), _NAME_TO_LEVEL.items())
     def test_log_level_number_added(self, level, number):
         """
         The log level number is added to the event dict.
@@ -476,7 +548,7 @@ class TestAddLoggerName:
         assert name == event_dict["logger"]
 
 
-def extra_dict() -> Dict[str, Any]:
+def extra_dict() -> dict[str, Any]:
     """
     A dict to be passed in the `extra` parameter of the `logging` module's log
     methods.
@@ -496,11 +568,11 @@ def extra_dict_fixture():
 
 class TestExtraAdder:
     @pytest.mark.parametrize(
-        "allow, misses",
+        ("allow", "misses"),
         [
             (None, None),
             ({}, None),
-            *[({key}, None) for key in extra_dict().keys()],
+            *[({key}, None) for key in extra_dict()],
             ({"missing"}, {"missing"}),
             ({"missing", "keys"}, {"missing"}),
             ({"this", "x_int"}, None),
@@ -509,9 +581,9 @@ class TestExtraAdder:
     def test_add_extra(
         self,
         make_log_record: Callable[[], logging.LogRecord],
-        extra_dict: Dict[str, Any],
-        allow: Optional[Collection[str]],
-        misses: Optional[Set[str]],
+        extra_dict: dict[str, Any],
+        allow: Collection[str] | None,
+        misses: set[str] | None,
     ):
         """
         Extra attributes of a LogRecord object are added to the event dict.
@@ -538,11 +610,11 @@ class TestExtraAdder:
         assert {} == actual
 
     @pytest.mark.parametrize(
-        "allow, misses",
+        ("allow", "misses"),
         [
             (None, None),
             ({}, None),
-            *[({key}, None) for key in extra_dict().keys()],
+            *[({key}, None) for key in extra_dict()],
             ({"missing"}, {"missing"}),
             ({"missing", "keys"}, {"missing"}),
             ({"this", "x_int"}, None),
@@ -550,9 +622,9 @@ class TestExtraAdder:
     )
     def test_add_extra_e2e(
         self,
-        extra_dict: Dict[str, Any],
-        allow: Optional[Collection[str]],
-        misses: Optional[Set[str]],
+        extra_dict: dict[str, Any],
+        allow: Collection[str] | None,
+        misses: set[str] | None,
     ):
         """
         Values passed in the `extra` parameter of the `logging` module's log
@@ -589,20 +661,18 @@ class TestExtraAdder:
     def _copy_allowed(
         cls,
         event_dict: EventDict,
-        extra_dict: Dict[str, Any],
-        allow: Optional[Collection[str]],
+        extra_dict: dict[str, Any],
+        allow: Collection[str] | None,
     ) -> EventDict:
         if allow is None:
             return {**event_dict, **extra_dict}
-        else:
-            return {
-                **event_dict,
-                **{
-                    key: value
-                    for key, value in extra_dict.items()
-                    if key in allow
-                },
-            }
+
+        return {
+            **event_dict,
+            **{
+                key: value for key, value in extra_dict.items() if key in allow
+            },
+        }
 
 
 class TestRenderToLogKW:
@@ -623,13 +693,42 @@ class TestRenderToLogKW:
 
         assert {"msg": "message", "extra": event_dict} == d
 
+    def test_handles_special_kw(self, event_dict):
+        """
+        "exc_info", "stack_info", and "stackLevel" aren't passed as extras.
+
+        Cf. https://github.com/hynek/structlog/issues/424
+        """
+        del event_dict["a"]  # needs a repr
+        event_dict["event"] = "message"
+
+        event_dict["exc_info"] = True
+        event_dict["stack_info"] = False
+        event_dict["stackLevel"] = 1
+
+        d = render_to_log_kwargs(None, None, event_dict)
+
+        assert {
+            "msg": "message",
+            "exc_info": True,
+            "stack_info": False,
+            "stackLevel": 1,
+            "extra": {
+                "b": [3, 4],
+                "x": 7,
+                "y": "test",
+                "z": (1, 2),
+            },
+        } == d
+
 
 @pytest.fixture(name="configure_for_processor_formatter")
 def _configure_for_processor_formatter():
     """
     Configure structlog to use ProcessorFormatter.
 
-    Reset both structlog and logging setting after the test.
+    Reset logging setting after the test (structlog is reset automatically
+    before all tests).
     """
     configure(
         processors=[add_log_level, ProcessorFormatter.wrap_for_formatter],
@@ -640,14 +739,13 @@ def _configure_for_processor_formatter():
     yield
 
     logging.basicConfig()
-    reset_defaults()
 
 
 def configure_logging(
     pre_chain,
     logger=None,
     pass_foreign_args=False,
-    renderer=ConsoleRenderer(colors=False),
+    renderer=ConsoleRenderer(colors=False),  # noqa: B008
 ):
     """
     Configure logging to use ProcessorFormatter.
@@ -740,7 +838,7 @@ class TestProcessorFormatter:
         If `pass_foreign_args` is `True` we set the `positional_args` key in
         the `event_dict` before clearing args.
         """
-        test_processor = call_recorder(lambda l, m, event_dict: event_dict)
+        test_processor = call_recorder(lambda _, __, event_dict: event_dict)
         configure_logging((test_processor,), pass_foreign_args=True)
 
         positional_args = {"foo": "bar"}
@@ -819,11 +917,11 @@ class TestProcessorFormatter:
         If non-structlog record contains exc_info, foreign_pre_chain functions
         have access to it.
         """
-        test_processor = call_recorder(lambda l, m, event_dict: event_dict)
+        test_processor = call_recorder(lambda _, __, event_dict: event_dict)
         configure_logging((test_processor,), renderer=KeyValueRenderer())
 
         try:
-            raise RuntimeError("oh noo")
+            raise RuntimeError("oh no")
         except Exception:
             logging.getLogger().exception("okay")
 
@@ -838,26 +936,26 @@ class TestProcessorFormatter:
         ProcessorFormatter should not have changed it.
         """
 
-        class MyException(Exception):
+        class MyError(Exception):
             pass
 
         def add_excinfo(logger, log_method, event_dict):
             event_dict["exc_info"] = sys.exc_info()
             return event_dict
 
-        test_processor = call_recorder(lambda l, m, event_dict: event_dict)
+        test_processor = call_recorder(lambda _, __, event_dict: event_dict)
         configure_logging(
             (add_excinfo, test_processor), renderer=KeyValueRenderer()
         )
 
         try:
-            raise MyException("oh noo")
+            raise MyError("oh no")
         except Exception:
             logging.getLogger().error("okay")
 
         event_dict = test_processor.calls[0].args[2]
 
-        assert MyException is event_dict["exc_info"][0]
+        assert MyError is event_dict["exc_info"][0]
 
     def test_other_handlers_get_original_record(self):
         """
@@ -907,7 +1005,7 @@ class TestProcessorFormatter:
         logger.handlers[0].setFormatter(formatter)
 
         try:
-            raise RuntimeError("oh noo")
+            raise RuntimeError("oh no")
         except Exception:
             logging.getLogger().exception("seen worse")
 
@@ -1026,15 +1124,11 @@ class TestProcessorFormatter:
         )
 
 
-@pytest.fixture(name="abl")
+@pytest_asyncio.fixture(name="abl")
 async def _abl(cl):
     return AsyncBoundLogger(cl, context={}, processors=[])
 
 
-@pytest.mark.skipif(
-    sys.version_info[:2] < (3, 7),
-    reason="AsyncBoundLogger is only for Python 3.7 and later.",
-)
 class TestAsyncBoundLogger:
     def test_sync_bl(self, abl, cl):
         """
@@ -1046,14 +1140,14 @@ class TestAsyncBoundLogger:
             CapturedCall(method_name="info", args=(), kwargs={"event": "test"})
         ] == cl.calls
 
-    @pytest.mark.asyncio
+    @pytest.mark.asyncio()
     async def test_protocol(self, abl):
         """
         AsyncBoundLogger is a proper BindableLogger.
         """
         assert isinstance(abl, BindableLogger)
 
-    @pytest.mark.asyncio
+    @pytest.mark.asyncio()
     async def test_correct_levels(self, abl, cl, stdlib_log_method):
         """
         The proxy methods call the correct upstream methods.
@@ -1063,14 +1157,11 @@ class TestAsyncBoundLogger:
         aliases = {"exception": "error", "warn": "warning"}
 
         alias = aliases.get(stdlib_log_method)
-        if alias:
-            expect = alias
-        else:
-            expect = stdlib_log_method
+        expect = alias if alias else stdlib_log_method
 
         assert expect == cl.calls[0].method_name
 
-    @pytest.mark.asyncio
+    @pytest.mark.asyncio()
     async def test_log_method(self, abl, cl):
         """
         The `log` method is proxied too.
@@ -1079,7 +1170,7 @@ class TestAsyncBoundLogger:
 
         assert "error" == cl.calls[0].method_name
 
-    @pytest.mark.asyncio
+    @pytest.mark.asyncio()
     async def test_exception(self, abl, cl):
         """
         `exception` makes sure 'exc_info" is set, if it's not set already.
@@ -1094,7 +1185,7 @@ class TestAsyncBoundLogger:
         assert ValueError is ei[0]
         assert ("omg",) == ei[1].args
 
-    @pytest.mark.asyncio
+    @pytest.mark.asyncio()
     async def test_exception_do_not_overwrite(self, abl, cl):
         """
         `exception` leaves exc_info be, if it's set.
@@ -1111,7 +1202,7 @@ class TestAsyncBoundLogger:
         ei = cl.calls[0].kwargs["exc_info"]
         assert (o1, o2, o3) == ei
 
-    @pytest.mark.asyncio
+    @pytest.mark.asyncio()
     async def test_bind_unbind(self, cl):
         """
         new/bind/unbind/try_unbind are correctly propagated.
@@ -1142,7 +1233,7 @@ class TestAsyncBoundLogger:
         assert {} == l5._context
         assert l4 is not l5
 
-    @pytest.mark.asyncio
+    @pytest.mark.asyncio()
     async def test_integration(self, capsys):
         """
         Configure and log an actual entry.
@@ -1166,4 +1257,26 @@ class TestAsyncBoundLogger:
             "level": "info",
         } == json.loads(capsys.readouterr().out)
 
-        reset_defaults()
+
+@pytest.mark.parametrize("log_level", [None, 45])
+def test_recreate_defaults(log_level):
+    """
+    Recreate defaults configures structlog and -- if asked -- logging.
+    """
+    logging.basicConfig(
+        stream=sys.stderr,
+        level=1,
+        force=True,
+    )
+
+    recreate_defaults(log_level=log_level)
+
+    assert BoundLogger is _CONFIG.default_wrapper_class
+    assert dict is _CONFIG.default_context_class
+    assert isinstance(_CONFIG.logger_factory, LoggerFactory)
+
+    log = get_logger().bind()
+    if log_level is not None:
+        assert log_level == log.getEffectiveLevel()
+    else:
+        assert 1 == log.getEffectiveLevel()
